use axum::{
    http::{StatusCode, header},
    response::{IntoResponse, Response},
};

use super::TAG;

const SOURCE_ARCHIVE: &[u8] = include_bytes!(concat!(env!("OUT_DIR"), "/archive.zip"));

/// Serve the source code archive as a downloadable zip file.
#[utoipa::path(
    get,
    path = "/code",
    tag = TAG,
    responses(
        (status = 200,
         description = "Source code archive as zip file",
         content_type = "application/zip",
         headers(
             ("content-disposition" = String, description = "Attachment filename"),
             ("content-length" = String, description = "File size in bytes")
         )
        ),
    )
)]
pub async fn route() -> Result<Response, StatusCode> {
    let content_length = SOURCE_ARCHIVE.len().to_string();
    let headers = [
        (header::CONTENT_TYPE, "application/zip"),
        (
            header::CONTENT_DISPOSITION,
            "attachment; filename=\"source.zip\"",
        ),
        (header::CONTENT_LENGTH, content_length.as_str()),
    ];

    Ok((headers, SOURCE_ARCHIVE).into_response())
}
