//! HTTP route definitions and router configuration.

use std::sync::Arc;

use tower_http::trace::TraceLayer;
use utoipa_axum::{router::OpenApiRouter, routes};

use super::state::AppState;
use crate::api::{code, query};

/// Create the main API router with all endpoints and middleware.
pub fn router(state: AppState) -> OpenApiRouter {
    let store = Arc::new(state);
    OpenApiRouter::new()
        .routes(routes!(query::route, code::route))
        .layer(TraceLayer::new_for_http())
        .with_state(store)
}
