//! Shared application state for HTTP handlers.

use crate::{storage::Postgres, text_encoder::TextEncoder, tokenize::Tokenizer};

/// Application state shared across all HTTP request handlers.
#[derive(Debug, Clone)]
pub struct AppState {
    /// Database connection pool.
    pub db: Postgres,
    /// Text tokenizer for processing queries and documents.
    pub tokenizer: Tokenizer,
    /// Primary embedding model for vector generation.
    pub embedder: TextEncoder,
    /// Reranking model for improving search relevance.
    pub reranker: TextEncoder,
    /// Text chunk size in words for processing.
    pub chunk_size: usize,
}
