//! File hashing utilities.

use std::{
    fs::File,
    io::{self, Read},
    path::Path,
};

use sha2::{Digest, Sha256};

/// Length of SHA256 hash in bytes.
pub const SHA256_LENGTH: usize = 32;

/// Compute SHA256 hash of a file.
pub fn file_sha256<P: AsRef<Path>>(path: P) -> io::Result<[u8; SHA256_LENGTH]> {
    let mut file = File::open(path)?;
    let mut hasher = Sha256::new();
    let mut buffer = [0; 1024];

    loop {
        let bytes_read = file.read(&mut buffer)?;
        if bytes_read == 0 {
            break;
        }
        hasher.update(&buffer[..bytes_read]);
    }

    let hash = hasher.finalize();
    let hash_bytes: [u8; SHA256_LENGTH] = hash.into();

    Ok(hash_bytes)
}
